<?php

App::uses('EvCaseStudiesAppController', 'EvCaseStudies.Controller');

class CaseStudiesController extends EvCaseStudiesAppController {

/**
 * Paginator defaults for admin.
 *
 * @var array
 */
	public $adminPaginate = [
		'contain' => [
			'CaseStudyCategory' => [
				'fields' => [
					'CaseStudyCategory.id',
					'CaseStudyCategory.name'
				]
			]
		]
	];

	protected function _adminPopulateLookups() {
		$this->loadModel('EvCaseStudies.CaseStudyCategories');
		$caseStudyCategories = $this->CaseStudyCategories->find('list');

		$this->set('caseStudyCategories', $caseStudyCategories);
	}

	public function beforeFilter() {
		parent::beforeFilter();

		$this->Auth->allow(array(
			'index',
			'view'
		));
	}

/**
 * Defines the columns displayed in the admin_index results table
 * using a columns whitelist.
 *
 * @return array containing the index columns
 */
	protected function _adminIndexColumns() {
		$indexColumns = parent::_adminIndexColumns();

		$Model = $this->{$this->modelClass};
		$modelAlias = $Model->alias;

		// add additional index columns
		$additionalIndexColumns = [
			'CaseStudyCategory.name' => [
				'label' => 'Category',
				'type' => 'string',
			]
		];

		return ArrayUtil::addAfter($indexColumns, "$modelAlias.name", $additionalIndexColumns);
	}

	protected function _adminFormFields() {
		$result = parent::_adminFormFields();

		$result['CaseStudy.body']['type'] = 'html';

		return $result;
	}

	/**
	 * index
	 *
	 * Displays all active Case Studies with pagination
	 */
	public function index() {
		$Model = $this->{$this->modelClass};

		// see if there is a linked content page
		$pageId = Configure::read('EvCaseStudies.page_id');

		if (!empty($pageId)) {
			$this->assignPage($pageId, 'page');
		}

		//Create database friendly now dateTime
		$nowTime = new DateTime();
		$nowTime->setTimezone(new DateTimeZone('UTC'));
		$nowTime = $nowTime->format('Y_m_d_H_i_s');

		$paginateParams = [
			'conditions' => array(
				'CaseStudy.is_active' => 1,
				'CaseStudy.published <=' => $nowTime
			),
			'order' => 'CaseStudy.sequence ASC',
			'contain' => array(
				'CaseStudyCategory',
				'FeaturedImage',
				'GalleryImage'
			),
			'limit' => Configure::read('EvCaseStudies.page_limit')
		];

		$params = $this->_processCaseStudyParams($this->request->params);

		$this->paginate = array(
			array_merge($paginateParams, $params)
		);

		$this->set('data', $this->paginate());

		// call the widgets
		$this->widgetLatestCaseStudies();
		$this->widgetCategories();
		$this->widgetFilterByCategory();
		$this->widgetFilterByDate();

		$this->view = 'EvCaseStudies.CaseStudies/index';
	}

	/**
	 * view
	 *
	 * Displays a specific case study based on it's id
	 * @param  int $id ID of the case study to display
	 */
	public function view($id) {
		$Model = $this->{$this->modelClass};

		$data = $Model->readForView($id);
		$this->Meta->set($data);

		$this->set('data', $data);

		// call the widgets
		$this->widgetLatestCaseStudies();
		$this->widgetCategories();

		$this->view = 'EvCaseStudies.CaseStudies/view';
	}

/**
 * Loop around the named portion of the supplied request parameters and append
 * requests to the current params array, in preparation for the database query
 *
 * @param array $requestParams Contains the current request parameters
 * @param array $currentParams Contains the current parameters
 * @return array Contains the combined named request and current parameters
 */
	protected function _processCaseStudyParams(array $requestParams = [], array $currentParams = []) {
		$ignoreParams = [
			'page',
			'named'
		];

		$dateParams = [
			'date',
			'startDate',
			'endDate'
		];

		$categoryIds = [];

		if (! empty($requestParams['named'])) {
			foreach ($requestParams['named'] as $param => $paramValue) {
				if (!in_array($param, $ignoreParams)) {

					//Check it isn't a date filter
					if (in_array($param, $dateParams)) {
						if ($param == 'startDate') {
							//Query for case studies published after this date
							$currentParams['conditions'][]['CaseStudy.published >='] = $paramValue;
						} elseif ($param == 'endDate') {
							//Query for case studies published before this date
							$currentParams['conditions'][]['CaseStudy.published <='] = $paramValue;
						}
					} else {
						$categoryIds[] = $paramValue;
					}

				}
			}
		}

		if (! empty($categoryIds)) {
			$currentParams['conditions']['CaseStudy.case_study_category_id'] = $categoryIds;
		}

		return $currentParams;
	}
}
